from enum import StrEnum


class Planet(StrEnum):
    Serpulo = 'serpulo'
    Erekir = 'erekir'


SectorNames = {
    Planet.Serpulo: {
        1: 'Atolls',
        3: 'Testing Grounds',
        15: 'Ground Zero',
        18: 'The Craters',
        20: 'Stained Mountains',
        21: 'Fungal Pass',
        23: 'Tar Fields',
        39: 'Weathered Channels',
        47: 'Sea Port',
        50: 'Frontier',
        54: 'Cruxscape',
        64: 'Facility 32M',
        # 68: 'Polar Aerodrome',
        81: 'Biomass Synthesis Facility',
        86: 'Frozen Forest',
        93: 'Planetary Launch Terminal',
        101: 'Salt Flats',
        108: 'Coastline',
        123: 'Desolate Rift',
        130: 'Nuclear Production Complex',
        134: 'Overgrowth',
        165: 'Extraction Outpost',
        210: 'Infested Canyons',
        213: 'Ruinous Shores',
        216: 'Naval Fortress',
        221: 'Tainted Woods',
        227: 'Impact 0078',
        246: 'Windswept Islands',
        260: 'Mycelia Bastion',
        264: 'Geothermal Stronghold',
    },
    Planet.Erekir: {
        3: 'Crevice',
        5: 'Karst',
        10: 'The Onset',
        12: 'Origin',
        14: 'Atlas',
        18: 'Stronghold',
        19: 'Split',
        25: 'Marsh',
        29: 'Basin',
        30: 'Peaks',
        36: 'Intersect',
        37: 'Crossroads',
        39: 'Ravine',
        41: 'Lake',
        43: 'Caldera',
        58: 'Siege',
        88: 'Aegis',
    },
}

ItemIds = {
    Planet.Serpulo: [
        'blast-compound',
        'coal',
        'copper',
        'graphite',
        'lead',
        'metaglass',
        'phase-fabric',
        'plastanium',
        'pyratite',
        'sand',
        'scrap',
        'silicon',
        'thorium',
        'titanium',
        'spore-pod',
        'surge-alloy',
    ],
    Planet.Erekir: [
        'beryllium',
        'carbide',
        # 'dormant-cyst',
        # 'fissile-matter',
        'graphite',
        'oxide',
        'phase-fabric',
        'thorium',
        'tungsten',
        'silicon',
        'surge-alloy',
    ]
}

ItemColors = {
    'beryllium': '#3a8f64',
    'blast-compound': '#ff795e',
    'carbide': '#89769a',
    'coal': '#53565c', # Real: #272727
    'copper': '#d99d73',
    # 'dormant-cyst': '#df824d',
    # 'fissile-matter': '#5e988d',
    'graphite': '#b2c6d2',
    'lead': '#8c7fa9',
    'metaglass': '#ebeef5',
    'oxide': '#e4ffd6',
    'phase-fabric': '#f4ba6e',
    'plastanium': '#cbd97f',
    'pyratite': '#ffaa5f',
    'scrap': '#777777',
    'sand': '#f7cba4',
    'thorium': '#f9a3c7',
    'titanium': '#8da1e3',
    'tungsten': '#768a9a',
    'silicon': '#53565c',
    'spore-pod': '#7457ce',
    'surge-alloy': '#f3e979',
}